#/usr/bin/python
# -*- coding: utf-8 -*-

import defaults as DEF

from ..utilities import vector, sceneFile
from utilities import col2bin, vcos, ZBuffer
from ..config import defaultpictureformat
from ..geoObjects.ray import Ray

from os import remove
from math import radians, tan
from scipy import cross
from PIL import Image

class Scene(object):
	def __init__(self, col = DEF.col, objs = [] , sun = None, ambient = None):
		self.col  = col
		self.objs = objs
		self.sun  = sun
		
		if ambient is None:
			if sun is None:
				self.ambient = vector(255)
			else:
				self.ambient = 0.5*vector(255)
		else:
			self.ambient = ambient
	
	def addObjects(self, *objs):
		self.objs.extend(objs)

	def setSun(self, sun):
		self.sun = sun

	def render(self, cam, res, filename = None):
		if filename is None:
			filename = '%s-%ix%i.%s' % (sceneFile.name, res[0], res[1], defaultpictureformat)
		
		print 'Now rendering %s with resolution %ix%i ...' % (filename, res[0], res[1])
		print
		
		x, y = res
		
		#Screen
		width = 2*cam.focalLength*tan(radians(cam.angle))
		hight = width/cam.aspectRatio
		
		p0 = cam.pos + cam.dir*cam.focalLength + .5*(-hight*cam.sky - width*cross(cam.dir, cam.sky))	# bottomleft point of screen
		
		upShift    = cam.sky*hight/(y-1)
		rightShift = width*cross(cam.dir, cam.sky)/(x-1)
		
		#Rendering
		backCol = col2bin(self.col)
		
		f = open('%s.tmp' % filename, 'w')
		
		for iy in range(y):
			for ix in range(x):
				p = p0 + iy*upShift + ix*rightShift
				zBuf = ZBuffer()
				screenRay = Ray(pos = p, dir = p-cam.pos)
				
				for obj in self.objs:
					distAndNorm = obj.geo.intersectionWithRay(screenRay).getDistAndNormOfNearestToRayStart()
					zBuf.append(distAndNorm, obj)
				
				if zBuf.obj is not None:					
					if self.sun is not None:						
						objRay = Ray(pos = screenRay.getPoint(zBuf.dist), dir = -self.sun.dir)
						
						directSunlight = True
						
						otherObjects = self.objs[:]
						del otherObjects[self.objs.index(zBuf.obj)]
							
						for otherObj in otherObjects:						
							if(otherObj.geo.intersectionWithRay(objRay).anyIsBeyondRayStart()):
								directSunlight = False
								break
						
						if directSunlight is True:
							f.write(col2bin( zBuf.obj.col * (self.sun.col/255.) * vcos(-self.sun.dir, zBuf.norm) ))
						else:
							f.write(col2bin(vector(0)))
					else:
						f.write(col2bin(zBuf.obj.col))
				else:
					f.write(backCol)
			
			print '\r%6.2f %%' % (100.*iy/y),		#progress
		print '\r100.00%'
		
		f.close()
		
		f = open('%s.tmp' % filename, 'r')
		
		im = Image.frombuffer('RGB', (x, y), f.read())
		im.save(filename)		
		
		f.close()
		remove('%s.tmp' % filename)		
